import json
from dataclasses import dataclass, field
from datetime import datetime
from typing import Any, Iterator, Optional

from turbofan.database import text  # type: ignore


@dataclass(frozen=True)
class Event:
    name: str
    data: dict
    id: Optional[int] = None
    created_at: Optional[datetime] = field(default_factory=datetime.utcnow)


class Events:
    """
    Events it's a list of events.
    """

    def __init__(self, db_session: Any):
        super().__init__()
        self._session = db_session

    def __len__(self) -> int:
        sql_stmt = """SELECT COUNT(*) FROM event"""
        prepared_statement = text(sql_stmt)
        row = self._session.execute(prepared_statement).first()
        return row[0]

    def append(self, event: Event) -> None:
        """
        Appends an event to the list of events.
        """
        sql_stmt = """INSERT INTO event (name, data, created_at) VALUES (:name, :data, :created_at)"""
        prepared_statement = text(sql_stmt)
        prepared_statement = prepared_statement.bindparams(
            name=event.name,
            data=json.dumps(event.data),
            created_at=event.created_at,
        )
        self._session.execute(prepared_statement)

    def __getitem__(self, event_id: int) -> Event:
        sql_stmt = """SELECT id, name, data, created_at FROM event WHERE id = :item"""
        prepared_statement = text(sql_stmt)
        prepared_statement = prepared_statement.bindparams(item=event_id + 1)
        row = self._session.execute(prepared_statement).first()
        if not row:
            raise KeyError

        return Event(id=row[0], name=row[1], data=json.loads(row[2]), created_at=row[3])

    def __iter__(self) -> Iterator[Event]:
        sql_stmt = """SELECT id, name, data, created_at FROM event"""
        prepared_statement = text(sql_stmt)
        results = self._session.execute(prepared_statement)
        for row in results:
            yield Event(
                id=row[0], name=row[1], data=json.loads(row[2]), created_at=row[3]
            )
